/* +--------------------------------------------------------------------
 *   TERMS OF USE: MIT License
 * +--------------------------------------------------------------------
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files
 * (the "Software"), to deal in the Software without restriction,
 * including without limitation the rights to use, copy, modify, merge,
 * publish, distribute, sublicense, and/or sell copies of the Software,
 * and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 * +--------------------------------------------------------------------
 
 build with propgcc 0.2.2 for single cog
 
 Reinhard	Jan. 2012
 
 */


#include "max147.h"
#include <propeller.h>
#include <stdio.h>

#define W 1060

#define Kp 	30
#define Kd  10
#define Ki 	2

#define BIAS 120

void pid ()
{
	short w,x,x2,e,e2,ei,y;
	w = W;
	e2 = 0;
	ei = 0;
	
	initMAX147();
	DIRA |= 0xFF;
	
	while(1)
	{
		x = readMAX147(0xFF);	// physical in
		e = w - x;				// error
		ei += e;				// "integrate" error
		y = BIAS + e * Kp + (e - e2) * Kd + ei * Ki;	// PID control output 
		e2 = e;					// store old error for next difference
		if( y > 255)			// check upper limit
			y=255;
		if(y < 0)				// check lower limit
			y=0;
		OUTA = y;				// physical out
	}
}

////////////////////////////////////////////////
// main returns never
////////////////////////////////////////////////
void main ()
{

	  pid ();
 
}